package com.loganmagnan.eventcore;

import com.loganmagnan.eventcore.managers.CooldownManager;
import com.loganmagnan.eventcore.managers.PlayerDataManager;
import com.loganmagnan.eventcore.managers.SpawnManager;
import com.loganmagnan.eventcore.managers.ChatManager;
import com.loganmagnan.eventcore.menusystem.PlayerMenuUtil;
import com.loganmagnan.eventcore.runnables.CooldownRunnable;
import com.loganmagnan.eventcore.utils.ClassRegistrationUtils;
import com.loganmagnan.eventcore.utils.ColorUtils;
import com.loganmagnan.eventcore.utils.Constants;
import com.loganmagnan.eventcore.utils.Utils;
import com.loganmagnan.eventcore.utils.command.CommandFramework;
import com.loganmagnan.eventcore.utils.config.FileConfig;
import com.loganmagnan.eventcore.utils.config.file.Config;
import lombok.Getter;
import lombok.Setter;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.plugin.java.JavaPlugin;
import java.util.HashMap;

@Getter
@Setter
public class EventCore extends JavaPlugin {

    // Main Class Instance
    @Getter private static EventCore instance;

    // Configuration Files
    private Config mainConfig;
    private FileConfig messagesConfig;

    // Managers
    private PlayerDataManager playerDataManager;
    private SpawnManager spawnManager;
    private ChatManager chatManager;
    private CooldownManager cooldownManager;

    // Menu System
    private HashMap<Player, PlayerMenuUtil> playerMenuUtilMap = new HashMap<>();

    // Command Framework
    private CommandFramework commandFramework = new CommandFramework(this);

    @Override
    public void onEnable() {
        instance = this;

        this.saveDefaultConfig();
        this.mainConfig = new Config("config", this);
        this.messagesConfig = new FileConfig(this, "messages.yml");

        new Constants();

        Bukkit.getConsoleSender().sendMessage(Utils.chatBar);
        Bukkit.getConsoleSender().sendMessage(ColorUtils.getMessageType("&dEventCore &7- &av" + this.getDescription().getVersion()));
        Bukkit.getConsoleSender().sendMessage(ColorUtils.getMessageType("&7Made by &eLoganM Development"));
        Bukkit.getConsoleSender().sendMessage(Utils.chatBar);

        this.loadCommands();
        this.loadManagers();
        this.loadListeners();
        this.loadRunnables();
    }

    @Override
    public void onDisable() {
        instance = null;
    }

    private void loadCommands() {
        ClassRegistrationUtils.loadCommands("com.loganmagnan.eventcore.commands");
    }

    private void loadManagers() {
        this.playerDataManager = new PlayerDataManager();
        this.spawnManager = new SpawnManager();
        this.chatManager = new ChatManager();
        this.cooldownManager = new CooldownManager();
    }

    private void loadListeners() {
        ClassRegistrationUtils.loadListeners("com.loganmagnan.eventcore.listeners");
    }

    private void loadRunnables() {
        this.getServer().getScheduler().runTaskTimerAsynchronously(this, new CooldownRunnable(), 0, 20);
    }

    public PlayerMenuUtil getPlayerMenuUtil(Player player) {
        PlayerMenuUtil playerMenuUtil;

        if (playerMenuUtilMap.containsKey(player)) {
            return playerMenuUtilMap.get(player);
        } else {
            playerMenuUtil = new PlayerMenuUtil(player);

            playerMenuUtilMap.put(player, playerMenuUtil);

            return playerMenuUtil;
        }
    }
}
